pub mod parser;

pub use crate::parser::body::*;
pub use crate::parser::*;

use std::io;
//use std::sync::Arc;

use tokio::net::TcpListener;
//use tokio::sync::RwLock;

const ADDR: &'static str = "0.0.0.0:7700";

#[tokio::main]
async fn main() {
    apiserver().await.unwrap();
    loop {
        std::thread::sleep(std::time::Duration::from_secs(1));
    }
}

pub async fn apiserver() -> io::Result<()> {
    let listener = match TcpListener::bind(ADDR).await {
        Ok(o) => o,
        Err(e) => {
            println!("error: {e}");
            std::process::exit(1);
        }
    };

    tokio::spawn(async move {
        loop {
            match listener.accept().await {
                Ok((socket, _remote_addr)) => {
                    let mut buf = vec![0; 1024];

                    match socket.readable().await {
                        Ok(_) => {
                            match socket.try_read(&mut buf) {
                                Ok(_) => match handle(&buf) {
                                    Some(o) => {
                                        let _ = socket.try_write(&o).unwrap();
                                    }
                                    None => {}
                                },
                                Err(e) => {
                                    println!("error: {e}");
                                    continue;
                                }
                            };
                        }
                        Err(e) => {
                            println!("error: {e}");
                            continue;
                        }
                    }
                }
                Err(err) => {
                    println!("error: {err}");
                }
            }
        }
    });
    Ok(())
}

pub fn handle(buf: &Vec<u8>) -> Option<Vec<u8>> {
    let mut rawdata = DataWrapper::new(buf.to_vec());
    let reply = match parse_inbound_msg(&mut rawdata) {
        Ok(o) => {
            println!("query: {}", o);
            println!("raw query: {:?}", o.to_raw());
            Message::set_reply(o)
        }
        Err(e) => {
            println!("parse inbound message error: {}", e);
            return None;
        }
    };

    match reply {
        Some(o) => {
            println!("reply: {}", o);
            println!("raw reply {:?}", o.to_raw());
            return Some(o.to_raw().into());
        }
        None => {
            return None;
        }
    }
}
